/*
 * Copyright (c) 2017-2018 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution  and  use  in  source  and binary  forms,  with  or  without
 * modification, are permitted provided that the following conditions are met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice and this list of conditions.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice and  this list of  conditions in the  documentation and/or
 *      other materials provided with the distribution.
 *
 * THE SOFTWARE  IS PROVIDED "AS IS"  AND THE AUTHOR  DISCLAIMS ALL WARRANTIES
 * WITH  REGARD   TO  THIS  SOFTWARE  INCLUDING  ALL   IMPLIED  WARRANTIES  OF
 * MERCHANTABILITY AND  FITNESS.  IN NO EVENT  SHALL THE AUTHOR  BE LIABLE FOR
 * ANY  SPECIAL, DIRECT,  INDIRECT, OR  CONSEQUENTIAL DAMAGES  OR  ANY DAMAGES
 * WHATSOEVER  RESULTING FROM  LOSS OF  USE, DATA  OR PROFITS,  WHETHER  IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR  OTHER TORTIOUS ACTION, ARISING OUT OF OR
 * IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 *                                           Anthony Mallet on Tue Feb 28 2017
 */
#include "acarduio.h"

#include <math.h>
#include <stdio.h>

#include "arduio_c_types.h"

#include "codels.h"


/* --- Function out1 ---------------------------------------------------- */

/** Codel ario_out1 of function out1.
 *
 * Returns genom_ok.
 * Throws arduio_e_sys.
 */
genom_event
ario_out1(uint8_t n, float value, const double ioscale[64],
          const arduio_conn_s *conn, const genom_context self)
{
  int s;
  uint16_t d;

  if (!conn) {
    arduio_e_baddev_detail d;
    snprintf(d.dev, sizeof(d.dev), "not connected to device");
    return arduio_e_baddev(&d, self);
  }

  d = (n < 64) ? value * ioscale[n] : 0;
  s = ario_send_msg(&conn->chan, "o%1%2", n, d);
  if (s) return ario_e_sys_error(NULL, self);

  return genom_ok;
}


/* --- Function out ----------------------------------------------------- */

/** Codel ario_out of function out.
 *
 * Returns genom_ok.
 * Throws arduio_e_sys.
 */
genom_event
ario_out(const sequence53_float *values, const double ioscale[64],
         const arduio_conn_s *conn, const genom_context self)
{
  uint16_t d[values->_length];
  size_t i, l;
  int s;

  if (!conn) {
    arduio_e_baddev_detail d;
    snprintf(d.dev, sizeof(d.dev), "not connected to device");
    return arduio_e_baddev(&d, self);
  }

  /* discard trailing nans */
  l = values->_length;
  while(l && isnan(values->_buffer[l-1])) l--;

  for(i = 0; i < l; i++)
    d[i] = (i < 64) ? values->_buffer[i] * ioscale[i] : 0;

  s = ario_send_msg(&conn->chan, "O%@", d, l);
  if (s) return ario_e_sys_error(NULL, self);

  return genom_ok;
}
