/*
 * Copyright (c) 2015-2018 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice and this list of conditions.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice and  this list of  conditions in the  documentation and/or
 *      other materials provided with the distribution.
 *
 *                                      Anthony Mallet on Mon Feb 16 2015
 */
#ifndef H_ARDUIO_CODELS
#define H_ARDUIO_CODELS

#include <errno.h>
#include <stdbool.h>
#include <string.h>
#include <termios.h>

#include "arduio_c_types.h"

struct ario_channel_s {
  char path[1024];
  int fd;

  uint8_t buf[256], r, w; /* read ring buffer */

  bool start;
  bool escape;
  uint8_t msg[256], len; /* last message */
};

struct arduio_conn_s {
  struct ario_channel_s chan;
};

static inline genom_event
ario_e_sys_error(const char *s, genom_context self)
{
  arduio_e_sys_detail d;
  size_t l = 0;

  d.code = errno;
  if (s) {
    strncpy(d.what, s, sizeof(d.what) - 3);
    l = strlen(s);
    strcpy(d.what + l, ": ");
    l += 2;
  }
  strerror_r(d.code, d.what + l, sizeof(d.what) - l);
  return arduio_e_sys(&d, self);
}

int	ario_open_tty(const char *device);
int	ario_wait_msg(const struct ario_channel_s *channel);
int	ario_recv_msg(struct ario_channel_s *chan, bool block);
int	ario_send_msg(const struct ario_channel_s *chan, const char *fmt, ...);

#endif /* H_ARDUIO_CODELS */
